/////////////////////////////// ///
// Organization : Mohamed Hamed //
// Eng : Mohamed Hamed         //
// Clock : 12 MHZ              //
////////////////////////////////
// LCD module connections
sbit LCD_RS at RC6_bit;
sbit LCD_EN at RC7_bit;
sbit LCD_D4 at RD4_bit;
sbit LCD_D5 at RD5_bit;
sbit LCD_D6 at RD6_bit;
sbit LCD_D7 at RD7_bit;

sbit LCD_RS_Direction at TRISC6_bit;
sbit LCD_EN_Direction at TRISC7_bit;
sbit LCD_D4_Direction at TRISD4_bit;
sbit LCD_D5_Direction at TRISD5_bit;
sbit LCD_D6_Direction at TRISD6_bit;
sbit LCD_D7_Direction at TRISD7_bit;
// End LCD module connections


//variables
char Freq_txt[11];
unsigned int tmr1_Val=0;
unsigned int OverNumb=0;
unsigned long Freq=0;
unsigned short tmr0_cnt=100;
bit flag;
char TPer_txt[15];
float TPer=0;

void _Init()
{
  trisd.f0=0;
  lcd_init();
  Lcd_Cmd(_LCD_CURSOR_OFF);
  Lcd_Cmd(_LCD_CLEAR);

  //timer0 config
  intcon.f7=1;   //enable global interrupt
  intcon.f6=1;  //enable pref interrupt
  intcon.f5=1; //timer0 overflow interrupt
  OPTION_REG.T0CS = 0; //Internal instruction cycle clock
  OPTION_REG.PSA = 0;  //0 = Prescaler is assigned to the Timer0 module
  OPTION_REG.PS2 = 1; OPTION_REG.PS1 = 1; OPTION_REG.PS0 = 1;  //prescalar is set to 256
  tmr0_cnt=100;
  tmr0=139; //Timer0 starts counting from 98(TMR0 Preload)
  

  //timer1 config
  PIE1.f0=1; //Enables the TMR1 overflow interrupt
  t1con.f1=1; //External clock (on the rising edge)
  t1con.f3=1; //Oscillator is shut-off
  t1con.f2=1; //Synchronize external clock input
  tmr1l=0; tmr1h=0; //clear timer1
}


//ISR
void interrupt()
{
  //delay_ms(1000)  ;
 if(intcon.f2==1) //if timer0 is overlfowed
 {
   if(tmr0_cnt--==1) //decrement and check tmr0_cnt until zer0 which means a second
   {   portb.f0= ~portb.f0;
     intcon.f7=0; t1con.f0=0;  //stop counting and global interrupts
     flag=1;  //set flag
   }
 //  portb.f1=~portb.f1;
   intcon.f2=0; //reset timer0 flag
   tmr0=139;  //Timer0 starts counting from 98 again
 }
 
 if(pir1.f0==1) //if timer1 is overlfowed
 {
   OverNumb++;  //increment number of overflow
   pir1.f0=0; //reset timer1 flag
 }
 
}




void main() 
{   trisb=0;
portb=0x00;
   _Init();
   
   lcd_out(1,1, "FrequencyMeter");
   for(;;)
   {
        if(flag==1) //if one second elapsed
        {
          tmr1_Val = (TMR1H << 8) + TMR1L; //get high and low from timer1
           Freq =  tmr1_Val;  //calculate frequency
           LongWordToStr(Freq,Freq_txt); //convert Frequency to string
           floatToStr(TPer,TPer_txt);   //convert Period to string
           lcd_out(2,1, Freq_txt); lcd_out_cp("Hz");  //display Frequency
           tmr1l=0; tmr1h=0; tmr1_Val=0;  OverNumb=0; Freq=0; TPer=0; //reset variables
           tmr0_cnt=100; flag=0; //reset variables

           intcon.f7=1; t1con.f0=1;  //reenabling counting and global interrupts
        }
   }
}




/*
To count more than 65536Hz using timer1 16bit use this formula:

Freq = (2^16  * OverflowNumber ) + timer1 count

*/

/*
To combine to 8bit into a 16 bit use:

TMR1_value = (TMR1H << 8) + TMR1L;

*/



/*
To calculate the PRELOAD value use this:

PRELOAD = 256 - ( Time you need * Frequency ) / ( 4 * Prescaler)

Example: if 10ms is needed we do this: 256-(0.010*16000000)/(4*256)= 99
so  99 should be the RTCC_PRELOAD value.

*/